#include "ProFile.h"

#include <CyberLib.h>                                   //библиотека-ускоритель скетчей
#include <LiquidCrystal_I2C.h>                          //библиотека дисплея LCD1602 i2c
#include <EEPROM.h>                                     //библиотека энергонезависимой памяти

#include "PID.h"

extern LiquidCrystal_I2C lcd;

// секция профиля
pr profile;
const byte SizeProfile = sizeof(pr);                            //длина поля данных
  
//чтение профиля из EEPROM
void loadProfile(uint8_t currentProfile)
{
  for (byte j = 0; j <= SizeProfile - 1; j++)
  {
    EEPROM.get((currentProfile - 1)*SizeProfile, profile);
  }
  return;
}

//сохранение профиля
void SaveProfile (uint8_t currentProfile)
{
  for (byte j = 0; j <= SizeProfile - 1; j++) {
    EEPROM.put((currentProfile - 1)*SizeProfile, profile);
  }
}

void initEeprom()                                      //Запись в память начальных значений
{
  if (EEPROM.read(500) == 'C')
  {
    //что-то только для уже инициализированой памяти
  } else {
    //профиль по умолчанию:
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("EEPROM");
    lcd.setCursor(0, 1);
    profile =
    {                                                     //температура низа максимальная
        6,   3,   5,   5,   5,   3,   6,                  //7 скоростей роста температуры шага *10
      100, 120, 100, 100,  30,  20, 120,                                     //7 продолжительностей шага
      100, 120, 160, 200, 230, 200,  20,                         //температуры верхнего нагревателя по 9 шагам
      5, 21, 25,                                                 //PID верха, Kp умноженный на 10, Ki Kd
      5, 21, 25,                                                 //PID низа, Kp умноженный на 10, Ki Kd
      100, 100                                            //ограничение мощности верха и низа в процентах
    };
    SaveProfile (1);
    profile =
    {                                                     //температура низа максимальная
        6,   3,   5,   5,   5,   3,   6,                  //7 скоростей роста температуры шага *10
      100,  30, 100, 100,  30,  20, 120,                                     //7 продолжительностей шага
      100, 120, 160, 200, 230, 200,  20,                         //температуры верхнего нагревателя по 9 шагам
      5, 21, 25,                                                 //PID верха, Kp умноженный на 10, Ki Kd
      5, 21, 25,                                                 //PID низа, Kp умноженный на 10, Ki Kd
      100, 100                                            //ограничение мощности верха и низа в процентах
    };
    SaveProfile (2);
    profile =
    {                                                     //температура низа максимальная
        6,   3,   5,   5,   5,   3,   6,                  //7 скоростей роста температуры шага *10
      100, 120, 100, 100,  40,  20, 120,                                     //7 продолжительностей шага
      100, 120, 160, 200, 240, 200,  20,                         //температуры верхнего нагревателя по 9 шагам
      5, 21, 25,                                                 //PID верха, Kp умноженный на 10, Ki Kd
      5, 21, 25,                                                 //PID низа, Kp умноженный на 10, Ki Kd
      100, 100                                            //ограничение мощности верха и низа в процентах
    };
    SaveProfile (3);
    profile =
    {                                                     //температура низа максимальная
        6,   1,   0,   0,   0,   0,   0,                  //7 скоростей роста температуры шага *10
      180, 0, 0, 0,  0,  0, 0,                                     //7 продолжительностей шага
      140, 140, 0, 0, 0, 0,  0,                         //температуры верхнего нагревателя по 9 шагам
      5, 21, 25,                                                 //PID верха, Kp умноженный на 10, Ki Kd
      5, 21, 25,                                                 //PID низа, Kp умноженный на 10, Ki Kd
      100, 100                                            //ограничение мощности верха и низа в процентах
    };
    SaveProfile (4);
    for (uint8_t i = 0; i < 11; i++)
    {
      lcd.print(".");
      delay(50);
      
    }
    EEPROM.write(500, 'C'); //Записываем метку что в памяти уже есть значения
  }
}

void EraseEeprom()
{
    lcd.clear();
    lcd.setCursor(0, 0);
    lcd.print("EEPROM formatted");
    for (int i = 0; i < 512; i++) {
        EEPROM.write(i, 255);
    }
} 
