#ifndef ProFile_H
#define ProFile_H

#include <arduino.h>

// секция профиля
// 7ть шагов пайки
// 1,2 предварительный нагрев /используется только НИ /
// 3,4 оплаввление  /выход по температуре ВИ/
// 5 пик  /выход по времени/
// 6 охлаждение /выход по температуре ВИ/
// 7 охлаждение /выход по температуре ВИ и НИ/
//╔═════╤══════════════════╤════╤═══════════╤══════════════════╗
//║ шаг │ НИ               │ ВИ │ Условие   │ Контроль времени ║
//╠═════╪══════════════════╪════╪═══════════╪══════════════════╣
//║ 0   │ +                │ -  │ t НИ      │ ДА               ║  нагрев низа
//╟─────┼──────────────────┼────┼───────────┼──────────────────╢
//║ 1   │ +                │ -  │ время     │ ДА               ║  нагрев низа
//╟─────┼──────────────────┼────┼───────────┼──────────────────╢
//║ 2   │ поддержание      │ +  │ t ВИ      │ ДА               ║  нагрев верха
//╟─────┼──────────────────┼────┼───────────┼──────────────────╢
//║ 3   │ поддержание      │ +  │ t ВИ      │ ДА               ║  нагрев верха
//╟─────┼──────────────────┼────┼───────────┼──────────────────╢
//║ 4   │ поддержание      │ +  │ время     │ ДА               ║  нагрев верха
//╟─────┼──────────────────┼────┼───────────┼──────────────────╢
//║ 5,6 │ -                │ -  │ t ВИ и НИ │  -               ║  охлаждение
//╚═════╧══════════════════╧════╧═══════════╧══════════════════╝

#define PROFILE_STEPS 7

struct pr {                // профиль
  byte rateStep[PROFILE_STEPS];    // скорость роста температуры максимальная скорость роста температуры умноженная на 10
  byte timeStep[PROFILE_STEPS];  // время перехода на следующий шаг 0 - 10часов 
  byte tempStep[PROFILE_STEPS]; // заданные температуры для шагов профиля ВИ
  byte kp1;                // пропорциональный коэффициент ВИ
  byte ki1;                // интегральный коэффициент ВИ
  byte kd1;                // дифференциальный коэффициент ВИ
  byte kp2;                // пропорциональный коэффициент НИ
  byte ki2;                // интегральный коэффициент НИ
  byte kd2;                // дифференциальный коэффициент НИ
  byte pwr_TOP;            // максимальная мощность верхнего нагревателя в %
  byte pwr_BOTTOM;         // максимальная мощность нижнего нагревателя в %
};


void loadProfile(uint8_t currentProfile);                       //чтение профиля из EEPROM
void SaveProfile (uint8_t currentProfile);                      //сохранение текущего профиля

void initEeprom();                                              //Запись в память начальных значений
void EraseEeprom();

#endif