
//27 октября 2021

//#pragma GCC optimize ("Os")                             //хитрая команда для компилятора
#pragma GCC optimize ("-O0")
//#pragma GCC push_options
//#pragma GCC pop_options

#include <Arduino.h>
#include <CyberLib.h>                                   //библиотека-ускоритель скетчей

#include "Global.h"

#include "LCD.h"
#include "PID.h"
#include "Profile.h"
#include "Keyboard.h"

#include <LiquidCrystal_I2C.h>                          //библиотека дисплея LCD1602 i2c

// структура статусов событий
#define REFLOW_STATE_IDLE              0x00      //экран в режиме простоя, выбор номера профиля
#define REFLOW_STATE_MENU_BOTTOM_PWR   0x12      //меню выбора мощности НИ редактируемого профиля
#define REFLOW_STATE_MENU_STEP_RAMP    0x13      //меню выбора скорости роста t° ВИ на конкретном шаге редактируемого профиля
#define REFLOW_STATE_MENU_STEP_TARGET  0x14      //меню выбора t° уставки ВИ на конкретном шаге редактируемого профиля
#define REFLOW_STATE_MENU_TOP_PWR      0x15      //меню выбора мощности ВИ редактируемого профиля
#define REFLOW_STATE_MENU_STEP_DWELL   0x16      //меню выбора времени удержания t° уставки ВИ на конкретном шаге редактируемого профиля
#define REFLOW_STATE_MENU_BOTTOM_P     0x17      //меню выбора коэффициента P для НИ редактируемого профиля
#define REFLOW_STATE_MENU_BOTTOM_I     0x18      //меню выбора коэффициента I для НИ редактируемого профиля
#define REFLOW_STATE_MENU_BOTTOM_D     0x19      //меню выбора коэффициента D для НИ редактируемого профиля
#define REFLOW_STATE_MENU_TOP_P        0x1A      //меню выбора коэффициента P для ВИ редактируемого профиля
#define REFLOW_STATE_MENU_TOP_I        0x1B      //меню выбора коэффициента I для ВИ редактируемого профиля
#define REFLOW_STATE_MENU_TOP_D        0x1C      //меню выбора коэффициента D для ВИ редактируемого профиля
// "процесс пайки"
#define REFLOW_STATE_STEP_INIT         0x21      //рост температуры с заданной для текущего шага ВИ скоростью
#define REFLOW_STATE_STEP_START        0x22
#define REFLOW_STATE_STEP_FLOW         0x23
#define REFLOW_STATE_STEP_COMPLETE     0x24
#define REFLOW_STATE_STEP_FREEZ        0x25
#define REFLOW_STATE_STEP_ERROR        0x26
#define REFLOW_STATE_STEP_FINISH       0x27

//byte rfState = REFLOW_STATE_STEP_INIT;
byte rfState = REFLOW_STATE_IDLE;                             //переменная статуса пайки

extern byte key;
byte keyLast=keyNONE;
extern byte keyCnt;

extern LiquidCrystal_I2C lcd;

extern pr profile;

byte currentProfile = 1;                                     //текущий профиль от 1 до 14
byte currentStep = 1;                                        //текущий шаг профиля

unsigned long curTic;
unsigned long prevTic;
boolean updateScreen = true;                                    //признак обновления экрана
byte cntKey,cntLCD=0;

unsigned long rfTargetTic; // максимальный порого времени
unsigned long stepPrevTic; //
byte rfPeriod; // период скорости роста температуры 1000/(profile.rateStep[currentStep] * 0.1) = 100/profile.rateStep[currentStep]
byte rfTargetTemp=0; // целевая темпиратура
double rfTempReal1=0, rfTempReal2=0; //реальная температура (измеренная max6675) ВИ и НИ соответственно

extern byte Output1, Output2;  // Мощьность на нигреватели
                        
byte rfTempStep1=0, rfTempStep2=0; // Значение темпиратуры ВИ и НИ
extern byte rfTempRound1, rfTempRound2; // Сглаженные значния

void sReset() {
  asm volatile ("jmp 0");
}

void OutputStop() {
    detachInterrupt(0);
    D12_Low; D11_Low;
    Output1=0;
    Output2=0;
    rfTempStep1=0;
    rfTempStep2=0;
}

void setup()
{
  Serial.begin(115200);

  //Описание пинов подключения
  D9_Out; D9_High;                              //cs MAX6675 датчика нижнего нагревателя
  D8_Out; D8_High;                              //cs MAX6675 датчика верхнего нагревателя
  D6_In; D6_High;                               //DO (SO) MAX6675
  D7_Out;                                       //CLK (SCK) MAX6675

  D12_Out; D11_Out;                             //выходы управления верхним и нижним нагревателями
  OutputStop();                                 //верхний и нижний выходы установить на выходах низкий уровень сигнала

  D2_In;                                        //настраиваем порт на вход для отслеживания прохождения сигнала через ноль
  D10_Out;                                      //пин пищалки buzzerPin = 10
  D4_Out; D4_Low;                               //внутренний вентилятор intefan выключаем
  D5_Out; D5_Low;                               //внешний вентилятор extefan выключаем

  lcd.begin();
  // Для дисплея спецсимвол Градус Цельсия ....
  lcd.createChar(0, GC);
  lcd.createChar(1, Th);
  lcd.createChar(2, Bh);
  //Мелодия приветствия
  beep_(200, 523);
  beep_(200, 659);
  beep_(200, 784);
  beep_(200, 1046);

  initEeprom();
  loadProfile(currentProfile);

  #if (DEBUGE==2)
    profile =
    {
       6,  3,    5,   5,   5,   3,   6,                                      //7 скоростей роста температуры шага *10
      100, 120, 100, 100,  30,  20, 120,                                     //7 продолжительностей шага
      100, 120, 160, 200, 230, 180,  20,                         //температуры верхнего нагревателя по 9 шагам
      //60, 120, 120, 40,  40,  10, 120,                                     //7 продолжительностей шага      
      //30, 60, 80, 100, 120, 140,  20,                         //температуры верхнего нагревателя по 9 шагам      
      5, 21, 25,                                                 //PID верха, Kp умноженный на 10, Ki Kd
      5, 21, 25,                                                 //PID низа, Kp умноженный на 10, Ki Kd
      100, 100                                                  //ограничение мощности верха и низа в процентах
    };
    Serial.println("Proile :");
    for (size_t i = 0; i < 7; i++) Serial.print(String(profile.rateStep[i])+ ",");
    Serial.println("");
    for (size_t i = 0; i < 7; i++) Serial.print(String(profile.timeStep[i])+ ",");
    Serial.println("");
    for (size_t i = 0; i < 7; i++) Serial.print(String(profile.tempStep[i])+ ",");
    Serial.println("");
    Serial.print("Kp1:"+String(profile.kp1));
    Serial.print("Ki1:"+String(profile.ki1));    
    Serial.println("Kd1:"+String(profile.kd1));        
    Serial.print("Kp2:"+String(profile.kp2));
    Serial.print("Ki2:"+String(profile.ki2));    
    Serial.println("Kd2:"+String(profile.kd2));        
    Serial.println("Pwr top:"+String(profile.pwr_TOP));        
    Serial.println("Pwr bottom:"+String(profile.pwr_BOTTOM));            
  #endif  

  #if (LOG==2)
      Serial.print(String(rfTempReal1,2)+";");
      Serial.print(String(rfTempReal2,2)+";");
      Serial.print(String(rfTempRound1)+";");
      Serial.print(String(rfTempRound2)+";");
  #endif  
}

// Измнение значния параметра
// true - изменения зделаны, переход на следующее newSate состояние
// false - если cancel или находимся в томже состоянии
byte ChangeValue(byte &value,byte min,byte max, byte newSate)
{
      if (key == keyUP) {
        if (value < max) value ++;
        else value = max;
        updateScreen = true;
      }

      if (key == keyDOWN) {
        if (value > min) value --;
        else value = min;
        updateScreen = true;
      }

      if (key == keyOK)  {
        updateScreen = true;
        keySetWaitRelease();
        rfState = newSate;
        return true;
      }

      if (key == keyLEFT) {
        updateScreen = true;
        rfState = REFLOW_STATE_IDLE;
      }
      return false;
}

byte F1=0;

extern uint16_t adc_key_in;

void loop()
{ Start

  curTic = millis();
  if ((prevTic-curTic > 100)) {
    cntKey++;
    F1++;
  }
  prevTic=curTic;

  if (cntKey>=2) {// каждые 200мс обработка клавиш
    float out_float;
    cntKey=0;
    key = keyGet();

    cntLCD++;

    switch (rfState) {// Обработка меню
    #include "mainMenuSetings.inc" // Меню настройки
    case REFLOW_STATE_IDLE:
      if (key == keyUP && keyCnt==2) { //если нажата кнопка вверх, нужно перейти к следующему профилю
        currentProfile = currentProfile + 1;
        if (currentProfile >= 15) currentProfile = 1; //если номер профиля = 14 и нажата кнопка вверх, нужно вернуться к профилю 1
        loadProfile(currentProfile);                                             //читаем из eeprom и перенастраиваем ПИД если изменили профиль
        updateScreen = true;
        keySetWaitRelease();
      }
      if (key == keyDOWN && keyCnt==2) {//то же, что и выше, просто спуститесь на один профиль
        currentProfile = currentProfile - 1;
        if (currentProfile <= 0) currentProfile = 14;
        loadProfile(currentProfile);                                            //читаем из eeprom и перенастраиваем ПИД если изменили профиль
        updateScreen = true;
        keySetWaitRelease();
      }

      if ((keyLast==keyOK)&&(key==keyNONE)&&(keyCnt>15)) { //держим "ОК" 5 секунд и заходим в меню настроек
        lcd.clear();
        lcd.setCursor(0, 0); lcd.print("Edit profile: ");lcd.print(currentProfile);
        lcd.setCursor(0, 1);lcd.print("Bottom PWR:    ");
        updateScreen = true;
        keySetWaitRelease();
        rfState = REFLOW_STATE_MENU_BOTTOM_PWR;
        return;
      }

      if (key == keyLEFT) {
        D4_Low;                                              //внутренний вентилятор выключить
        D5_Low;                                              //внешний вентилятор выключить
      }

      //Удерживаем keyOK 8сек Стираем всю память и переходим в начало
      if ((key==keyOK)&&(keyCnt>40)) {
        EraseEeprom();
        sReset();
      }

      //включаем пайку, кнопка сработает после отпускания "ОК" если нажатие меньше 3х секунд
      if ((keyLast==keyOK)&&(key==keyNONE)) {
        beep_(500, 1045);                                         //звуковой сигнал при старте профиля
        initPid();
        rfState = REFLOW_STATE_STEP_INIT;
      }
      break;
    case REFLOW_STATE_STEP_FINISH:
      if ((keyLast==keyOK)&&(key==keyNONE)) {
        keySetWaitRelease();
        updateScreen = true;
        rfState = REFLOW_STATE_IDLE;
      }
    default:
      if ((rfState>=REFLOW_STATE_STEP_INIT)&&(key == keyLEFT)) {
        lcd.clear();
        lcd.setCursor(0, 1);  lcd.print("CANCEL");
        updateScreen = true;
        rfState = REFLOW_STATE_STEP_ERROR;
      }
    }
    if (key!=keyLast) {
       keyLast=key;
    }
  }

  // Обработка состояния и режимов работы 
  if (((rfState & 0x20) || (rfState==REFLOW_STATE_IDLE)) && cntLCD >3) {

    // Чтение датчиков термопар и вывод данных по текущей темпиратуре излучателей
    rfTempReal1 = max6675_read_temp(0);
    rfTempRound1=rfTempRound1 * 0.6 + 0.4 * rfTempReal1;

    rfTempReal2 = max6675_read_temp(1);
    rfTempRound2=rfTempRound2 * 0.6 + 0.4 * rfTempReal2;
     
    lcdTemp();
    cntLCD=0;
  
    // Проверка на аварию ДОДЕЛАТЬ !
    if ((rfTempRound1>250) || (rfTempRound2>250)) {
      //rfState=REFLOW_STATE_STEP_ERROR;
    }

    #if (LOG==1)
      Serial.print("Step:" + String(currentStep));
      Serial.print(" R1:" + String(rfTempReal1,2));
      Serial.print(" R2:" + String(rfTempReal2,2));
      Serial.print(" t1:" + String(rfTempRound1));
      Serial.println(" t2:" + String(rfTempRound2));
    #endif  

    switch (rfState) {
      case REFLOW_STATE_IDLE:
        if (updateScreen) { // Обновляем дисплей в режиме ожидания
          lcd.clear();
          lcd.setCursor(0, 0);lcd.print("IDLE");
          lcd.setCursor(0, 1);lcd.print("P");lcd.print(currentProfile);
          updateScreen = false;
        }
        break;
      case REFLOW_STATE_STEP_INIT:
        lcd.clear();
        lcd.setCursor(0, 0);lcd.print("St");
        lcd.setCursor(0, 1);lcd.print("P");lcd.print(currentProfile);
        rfTempStep2 = rfTempReal2; // текущая тмпиратура НИ как начальная
        /*
          CHANGE – прерывание вызывается при любом изменении значения на входе;
          RISING – вызов прерывания при изменении уровня напряжения с низкого (Low) на высокий(HIGH)
          FALLING – вызов прерывания при изменении уровня напряжения с высокого (HIGH) на низкий (Low)
        */
        attachInterrupt(0, Dimming, RISING);          //настроить срабатывание прерывания interrupt0 на pin 2
        //attachInterrupt(0, detect, FALLING);        //настроить срабатывание прерывания interrupt0 на pin 2 на низкий уровень
        currentStep = 0;
        rfState = REFLOW_STATE_STEP_START;
        break;

      case REFLOW_STATE_STEP_START:
        lcd.setCursor(2, 0);lcd.print(currentStep+1); //выводим номер шага

        rfPeriod = 100 / (profile.rateStep[currentStep]); // период скорости роста температуры 1000/(profile.rateStep[currentStep - 1] * 0.1)
        rfTargetTemp = profile.tempStep[currentStep]; // целевая темпиратура

        if (currentStep<2) {lcd.setCursor(8, 1);lcd.print(rfTargetTemp);lcd.write(byte(0));}
        else {lcd.setCursor(8, 0);lcd.print(rfTargetTemp);lcd.write(byte(0));}
        
        rfTargetTic = profile.timeStep[currentStep];
        if (rfTargetTic!=0) rfTargetTic = curTic + rfTargetTic*1000; // максимальный порого времени
        else rfTargetTic = curTic + 36000*1000; // 0 бесконечность (10Часов)
        if (currentStep>=5) { // Охлаждение
          OutputStop();
          lcd.setCursor(8, 1);lcd.print(rfTargetTemp);lcd.write(byte(0));
          lcd.setCursor(8, 0);lcd.print(rfTargetTemp);lcd.write(byte(0));
          rfState = REFLOW_STATE_STEP_FREEZ;
          break;
        }
        if (currentStep==2) rfTempStep1=rfTempReal1; // Для шага 2 отталкиваемся от реальной темпиратуры ВИ
        rfState = REFLOW_STATE_STEP_FLOW;
        break;

      case REFLOW_STATE_STEP_FLOW:
        if ( (curTic - stepPrevTic) > rfPeriod) { // Период для следующего шага расчета температуры
          stepPrevTic = curTic;
          if (currentStep<2) { // Увеличиваем темпиратуру
            if (rfTempStep2 < rfTargetTemp) rfTempStep2++;            
          } else {
            if (rfTempStep1 < rfTargetTemp) rfTempStep1++;
          }
        }

        #if (LOG==2)
          Serial.print(String(rfTempReal1,2)+";");
          Serial.print(String(rfTempReal2,2)+";");
          Serial.print(String(rfTempRound1)+";");
          Serial.print(String(rfTempRound2)+";");
        #endif  

        #if (LOG==2)
          Serial.print("1;");
        #endif  
        Output1 = Pid1(rfTempRound1, rfTempStep1);

        #if (LOG==2)
          Serial.print("2;");
        #endif  
        Output2 = Pid2(rfTempRound2, rfTempStep2);

        #if (LOG==2)
          Serial.print(String(rfTargetTemp)+";");
          Serial.print(String(rfTempStep1)+";");
          Serial.print(String(rfTempStep2)+";");          
          Serial.print(String(Output1)+";");          
          Serial.print(String(Output2)+";");                              
        #endif  

        // Проверка по действительной температуре
        if ((currentStep == 0) && (rfTempRound2 >= rfTargetTemp-5)) { // НИ достиг температуры
            rfState = REFLOW_STATE_STEP_COMPLETE;
            break;
        }
        if (((currentStep == 2) || (currentStep == 3)) && (rfTempRound1 >= rfTargetTemp-5)) { // ВИ достиг температуры
            rfState = REFLOW_STATE_STEP_COMPLETE;
            break;
        }
        // Проверка по времени
        if (curTic>rfTargetTic) {
          if ((currentStep == 1)&&(rfTempRound2 >= rfTargetTemp-5)) { // НИ достиг температуры и времени
            rfState = REFLOW_STATE_STEP_COMPLETE;
          } else  if (currentStep == 4) { // Чип можно снимать
            OutputStop();          
            beep_(500, 300);//звуковой сигнал
            rfState = REFLOW_STATE_STEP_COMPLETE;
          } else { // ошибка: время вышло на темпиратурный режим не вышли
            lcd.clear();
            lcd.setCursor(0, 0);lcd.print("St");lcd.print(currentStep+1);
            lcd.setCursor(0, 1);lcd.print("ERROR OUT");
            rfState = REFLOW_STATE_STEP_ERROR;
            Serial.println("Time out");
          }
          break;
        }

        // Выводи картинки излучателей ВИ и НИ
        // верхний нагреватель
        lcd.setCursor(4, 0);
        if (currentStep>=2 && currentStep<=4) {int b=(rfTargetTic-curTic)/1000; lcd.print(b); lcd.print(" "); }
        else lcd.print("   ");
        // нижний нагреватель выводим время
        lcd.setCursor(4, 1);
        if (currentStep<2) {
          int b=(rfTargetTic-curTic)/1000;
          if ((b>0)&&(b<999)) { lcd.print(b); lcd.print(" ");}
          else {lcd.write(byte(2));lcd.write(byte(2));lcd.write(byte(2));}
        } else if (currentStep<5) {lcd.write(byte(2));lcd.write(byte(2));lcd.write(byte(2));}
        else lcd.print("   ");
        break;
      case REFLOW_STATE_STEP_COMPLETE:
        if (currentStep == 6) {
          lcd.clear();
          lcd.setCursor(0, 0);  lcd.print("COMPLITE");
          // мелодия окончания пайки
          for (uint8_t j = 1; j < 5; j++) {
            beep_(100, 1046);
            delay(100);
          }
          rfState = REFLOW_STATE_STEP_FINISH;
          break;
        }
        currentStep++;
        rfState = REFLOW_STATE_STEP_START;
        break;
      case REFLOW_STATE_STEP_FREEZ:
        if ( (curTic - stepPrevTic) > rfPeriod) { // Период для следующего шага расчета температуры
          stepPrevTic = curTic;
          if (rfTempStep1 > rfTargetTemp) rfTempStep1--; // Уменьшаем темпиратуру
          if (rfTempStep2 > rfTargetTemp) rfTempStep2--; // Уменьшаем темпиратуру
        }

        // Проверка по действительной температуре
        if (rfTempRound1 < rfTargetTemp) { // ВИ достиг температуры остывания
          rfState = REFLOW_STATE_STEP_COMPLETE;
          break;
        }
        // Проверка по времени
        if (curTic>rfTargetTic) {
          if (currentStep == 6) { // не грубая ошибка
            rfState = REFLOW_STATE_STEP_COMPLETE;
          } else { // ошибка: время вышло на темпиратурный режим не вышли
            rfState = REFLOW_STATE_STEP_ERROR;
          }
        }
        break;
      case REFLOW_STATE_STEP_ERROR:
        OutputStop();
        keySetWaitRelease();
        beep_(200, 523);
        beep_(200, 659);
        beep_(200, 784);
        beep_(200, 1046);
        rfState = REFLOW_STATE_STEP_FINISH;
        break;
      case REFLOW_STATE_STEP_FINISH:
        beep_(200, 523);
        break;
    }
    delay(50);
    #if (LOG==2)    
      Serial.println(" ");
    #endif  
  } else {
    delay(40);
  }
  End
}